==> apache-skywalking-go-0.4.0-src.tgz: apache-skywalking-go-0.4.0/plugins/grpc/server_sendresponse_interceptor.go <==
// Licensed to Apache Software Foundation (ASF) under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Apache Software Foundation (ASF) licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package grpc

import (
	"github.com/apache/skywalking-go/plugins/core/operator"
	"github.com/apache/skywalking-go/plugins/core/tracing"
)

type ServerSendResponseInterceptor struct {
}

func (h *ServerSendResponseInterceptor) BeforeInvoke(invocation operator.Invocation) error {
	if tracing.ActiveSpan() == nil {
		return nil
	}
	cs := invocation.Args()[1].(*nativeStream)
	method := cs.Method()
	s, err := tracing.CreateLocalSpan(formatOperationName(method, "/Server/Response/SendResponse"),
		tracing.WithLayer(tracing.SpanLayerRPCFramework),
		tracing.WithTag(tracing.TagURL, method),
		tracing.WithComponent(23),
	)
	if err != nil {
		return err
	}
	invocation.SetContext(s)
	return nil
}

func (h *ServerSendResponseInterceptor) AfterInvoke(invocation operator.Invocation, result ...interface{}) error {
	if invocation.GetContext() == nil {
		return nil
	}
	span := invocation.GetContext().(tracing.Span)
	if err, ok := result[0].(error); ok && err != nil {
		span.Error(err.Error())
	}
	span.End()
	return nil
}

==> apache-skywalking-go-0.5.0-src.tgz: apache-skywalking-go-0.5.0/plugins/grpc/server_sendresponse_interceptor.go <==
// Licensed to Apache Software Foundation (ASF) under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Apache Software Foundation (ASF) licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.

package grpc

import (
	"github.com/apache/skywalking-go/plugins/core/operator"
	"github.com/apache/skywalking-go/plugins/core/tracing"
)

type ServerSendResponseInterceptor struct {
}

func (h *ServerSendResponseInterceptor) BeforeInvoke(invocation operator.Invocation) error {
	if tracing.ActiveSpan() == nil {
		return nil
	}
	var cs *nativeStream
	if arg1, ok := invocation.Args()[1].(*nativeStream); ok {
		cs = arg1
	} else if arg2, ok := invocation.Args()[2].(*nativeStream); ok {
		cs = arg2
	} else {
		return nil
	}

	method := cs.Method()
	s, err := tracing.CreateLocalSpan(formatOperationName(method, "/Server/Response/SendResponse"),
		tracing.WithLayer(tracing.SpanLayerRPCFramework),
		tracing.WithTag(tracing.TagURL, method),
		tracing.WithComponent(23),
	)
	if err != nil {
		return err
	}
	invocation.SetContext(s)
	return nil
}

func (h *ServerSendResponseInterceptor) AfterInvoke(invocation operator.Invocation, result ...interface{}) error {
	if invocation.GetContext() == nil {
		return nil
	}
	span := invocation.GetContext().(tracing.Span)
	if err, ok := result[0].(error); ok && err != nil {
		span.Error(err.Error())
	}
	span.End()
	return nil
}
